C///////////////////////////////////////////////////////////////////
C
C            PSR: PERFECTLY STIRRED REACTOR
C
C     WRITTEN BY:
C         ZHUYIN REN (Laniu) & Stephen B. Pope 
C         MAE, Cornell University, 14850
C         Jan 4, 2004  
C
C COMMENTS: 
C The current available PSR code developed at Sandia is unable to realiably predict the
C unstable branches of the solution. Here a new Fortran code based on arc-length continuation
C in the composition(mass fractions)-frequence-temperature space is developed and 
C demonstrated. The new method is capable of predicting the combustion curve accurately and    
C is able to treat cases with heat loss. 
C
C The steady-state PSR equations solved are:
C 
C  0=-\omega ( Y- Y^{in})+ S(Y),    where Y is the vector of mass fractions (KK)
C  0=-\omega (h-h^{in})-Q_{LOSS}/(\rho V),
C
C where: omega is the inverse residence time; Y is the vector of mass fractions; 
C        S its rate of change due to chemical reactions; h is specific enthalpy;
C        Q_{LOSS} is the heat transfer rate from the reactor; rho is density;
C        and V is the volume of the reactor.
C
C*******************************************************
C Input files:
C   chem.bin needed by chemkin
    
C    psr.inp: SPEC(moles), PRES(atm), VOL($cm^3$), TINL(K), QLOS(Cal/Sec) (Necessary);
C             RESO, NMAX, OMEI(1/Sec), OMEN(1/Sec) (Optional). 
C             Default values:  RESO 0.2; OMEINI 1.d-2; NMAX 3000; OMEND=1.d10

C Output files: 
C    psr.out:    main output, the first column is omega, and the second column is temperature.
C    ddasac.out: error log about DDASAC.
C    psrini.out: initialization information.

C Output on sreen:
C  Current successful points;  Frequency;  Temperature; current acrlength step;     
C
C*********************************************************
C Tips:
C  1. For the heat loss cases, it may be necessary to adjust OMEINI to get the first stable
C     point. The default value is OMEINI=1.d-2.
C  2. To get better resolution, you can decrease the value of RESOL. The default value is RESOL=0.2.
C  3. Possible failures may be caused by the initialization of DDASAC and bad approximation for
C     ${\bf g}$. You can solve this problem by changing the DDASAC error tolerance or
C     increasing the resolution (decrease the value of RESOL) .
C 
C*********************************************************
C Method:
C  Arc-length continuation.  The variables Y, h, T, omega are considered as functions
C  of the arclength, s.  From a consistent initial condition, the equations solved are:
C 
C  0=-\omega ( Y- Y^{in})+ S(Y),    where Y is the vector of mass fractions (KK)
C  0=-\omega (h-h^{in})-Q_{LOSS}/(\rho V),
C  dZ/ds=1,  
C  0=Z- g'*X,
C  where X=[ Y/Y^{ref}, T/T^{ref}, \ln(\omega/\omega^{ref})] and  g=(X^k- X^{k-1})/(s_k- s_{k-1}),
C  where k and k-1 denote the previous two computed points.
C The default value for the reference values are 1. And we employ ddasac to solve the above DAEs.

C/////////////////////////////////////////////////////////////////////
C
      PROGRAM PSR 
C
C*****precision > double

      IMPLICIT DOUBLE PRECISION (A-H, O-Z), INTEGER (I-N)
C*****END precision > double
       
      include 'param.com' 
      
      CHARACTER CWORK(LENCWK)*16   
      CHARACTER KSYM(KMAX)*16, ENAME(MMAX)*16 
      DIMENSION XINL_EQ(KMAX), YINL_EQ(KMAX) 
      DIMENSION Y_EQ(KMAX), U1(KMAX+2), U2(KMAX+2),U3(KMAX+2)
      
C************CONTENT OF param.com**********
C      PARAMETER (LENRWK=10000, LENIWK=10000,  LENCWK=100,
C     1           LIN=5, LOUT=6, MMAX=8,KMAX=60,
C     2           LINKCK=25)
C       DIMENSION RWORK(LENRWK), IWORK(LENIWK)
C       DIMENSION WT(KMAX), XINL(KMAX), YINL(KMAX) 
C         COMMON /PARAM/ IWORK, RWORK, WT,XINL,YINL
C************CONTENT OF param.com**********

C*************PARAMETERS AND ARRAYS****************
C    RWORK    -REAL WORKSPACE FOR CHEMKIN 
C    IWORK    -INTEGER WORKSPACE FOR CHEMKIN
C    CWORK    -CHARACTER WORKSPACE FOR CHEMKIN
C    LIN/LOUT/LINKCK -I/O OUPUT
C     MM      -NUMBER OF ELEMENTS
C    MMAX     - MAXIMUM NUMBER OF ELEMENTS 
C     KK      - NUMBER OF SPECIES
C    KMAX     - MAXIMUM NUMBER OF SPECIES
C     II      - NUMBER OF REACTIONS
C    PATM AND RU- ATMOSPHERE PRESSURE AND GAS CONSTANTS (CM-G-S) 
C      WT     - MOLECULAR WEIGHTS   
C      ENAME  - ELEMENTS NAMES
C     KSYM    - SPECIES  NAMES
C                   Data type - CHARACTER*(*) array
C                   Dimension KNAME(*) at least KK,      
C     XINL    - MOLE FRACTIONS OF THE INLET STREAM.
C     YINL    - MASS FRACTIONS OF THE INLET STREAM.
C     TINL    - TEMPERATURE OF THE INLET STREAM.
C     XINL_EQ - MOLE FRACTIONS OF THE EQUILIBRIUM COMPOSITION OF THE INLET STREAM.
C     YINL_EQ - MASS FRACTIONS OF THE EQUILIBRIUM COMPOSTION OF THE INLET STRAM.
C      OMEINI - START FREQUENCE POINT OF THE CURVE. 
C       Y_EQ  - EQUILIBRIUM COMPOSITION WITH HEAT LOSS WHEN OMEINI.
C       T_EQ  - EQUILIBRIUM TEMPERATURE WITH HEAT LOSS WHEN OMEINI.    
C       U1    - COMPOSITION OF THE FIRST POINT OF THE CURVE (OMEINI, KK MASS FRACTIONS,ENTHALPY)
C       U2    - COMPOSITION OF THE SECOND POINT OF THE CURVE (10*OMEINI, KK MASS FRACTIONS, ENTHALPY)
C       P     - PRESSURE (CM-G-S)
C       V     - VOLUME OF THE REACTOR (CM^3)
C       QLOSS - HEAT LOSS OF THE REACTOR (CM-G-S) 
C      OMEND -User_specified ending point of the FREQUENCE      
C       P, V, QLOSS AND OMEND ARE READ FROM THE INPUT FILE PSR.INP
C      NOTE: QLOS IN PSR.INP IS IN CAL/SEC UNITS, IN THE CODE WE CONVERT TO
C           CM-G-S UNITS.
C****************************************************************

C*************WRITE VERSION NUMBER************
C
      WRITE (LOUT,15)
15     FORMAT (
     1/' PSR:  Perfectly Stirred Reactor Code',
     2/'      Jan. 2004, Cornell University',
     3/'       DOUBLE PRECISION')

C*********READ IN THE CHEMKIN INPUT FILE***********

      OPEN (LINKCK, FORM='UNFORMATTED',STATUS='UNKNOWN',
     1              FILE='chem.bin')

C ***INITIALIZE THE CHEMKIN AND WORKSPACES*********

       CALL PSRINI (LINKCK, LOUT, MM, MMAX, KK, KMAX, II, RU, PATM, WT, 
     1           KSYM, ENAME, LENIWK, LENRWK, LENCWK, IWORK, RWORK, 
     2          CWORK)

C**** READ IN THE KEYWORD PARAMETERS FROM THE INPUT FILE PSR.INP.*****
   
        CALL PSRKEY (LIN, LOUT, MM, KK,II, RU, PATM, WT(1:KK),
     1        KSYM(1:KK), ENAME(1:MM), IWORK, RWORK, CWORK,
     2        XINL(1:KK), YINL(1:KK), TINL, P, V, QLOSS, OMEINI,OMEND,
     3        NITERMAX, RESOL)

C**** GET THE EQUILIRIUM COMPOSITION OF THE INLET STREAM BY CALLING THE CEQ SUBROUTINE BY POPE ****

        CALL PSREQUI(LIN, LOUT, MM, KK, WT(1:KK),
     1        KSYM(1:KK), ENAME(1:MM), IWORK, RWORK, CWORK,
     2        XINL(1:KK), YINL(1:KK), TINL, P, XINL_EQ(1:KK),
     3        YINL_EQ(1:KK), TINL_EQ)    
          
C*** GET THE EQUILIBRIUM COMPOSITION OF THE PSR FOR A USER-SPECIFIED SMALL OMEINI
C CONSIDERING THE HEAT LOSS OF THE REACTOR BY INTEGRATING THE GOVERNING ODES TO STEADY
C STATE USING DVODE.****

        CALL PSREQUIDVODE(LINKCK,LIN, LOUT, MM, KK, WT(1:KK),
     1        KSYM(1:KK), IWORK, RWORK, CWORK,
     2        TINL, YINL(1:KK),YINL_EQ(1:KK), TINL_EQ, P,V,QLOSS,
     3        OMEINI, Y_EQ(1:KK), T_EQ)
      
        
C***USE DDASAC TO GET THE FIRST TWO POINTS ON THE CURVE (AT OMEINI AND 10*OMEINI) BY 
C SOLVING THE CORRESPONDING DAES***.
       
        CALL PSRSOLINI(LIN, LOUT, MM, KK, WT(1:KK),
     1        KSYM(1:KK), IWORK, RWORK, CWORK,
     2        TINL,Y_EQ(1:KK), T_EQ, P,V, QLOSS,OMEINI,
     3        U1, U2,U3)        
      

       
C***USE  DDASAC to GET THE CURVE BY SOLVING THE CORRESPONDING DAES ALONG THE ARCLENGTH.       
       CALL PSRSOL( MM, KK, KSYM(1:KK), CWORK,
     1        TINL,Y_EQ(1:KK), T_EQ, P,V, QLOSS,OMEND,
     2        U1, U2,U3, NITERMAX, RESOL)  

       
      
      STOP
      END




